Adds the CPPALLY_PRESERVE_ALTREP flag to Makevars
Source:R/use_preserve_altrep.R
use_preserve_altrep_flag.RdAdds a flag to Makevars which enables lazy materialisation of ALTREP vectors.
R/use_preserve_altrep.R
use_preserve_altrep_flag.RdAdds a flag to Makevars which enables lazy materialisation of ALTREP vectors.