R/use_cppally.R
use_cppally.Rd
usethis style helper to add the necessary setup to a new package to help users get started with writing C++ code.
use_cppally()
Invisibly sets up the necessary conditions for developing a package with cppally.